package com.example.flutter_bk_ua

import android.app.UiModeManager
import android.content.Context
import android.content.pm.PackageInfo
import android.content.pm.PackageManager
import android.content.res.Configuration
import android.os.Build
import androidx.annotation.NonNull
import io.flutter.embedding.engine.plugins.FlutterPlugin
import io.flutter.plugin.common.MethodCall
import io.flutter.plugin.common.MethodChannel
import io.flutter.plugin.common.MethodChannel.MethodCallHandler
import io.flutter.plugin.common.MethodChannel.Result

/** FlutterBkUseragentPlugin */
class FlutterBkUAPlugin: FlutterPlugin, MethodCallHandler {
  /// The MethodChannel that will the communication between Flutter and native Android
  ///
  /// This local reference serves to register the plugin with the Flutter Engine and unregister it
  /// when the Flutter Engine is detached from the Activity
  private lateinit var channel : MethodChannel
  private lateinit var context: Context

  override fun onAttachedToEngine(@NonNull flutterPluginBinding: FlutterPlugin.FlutterPluginBinding) {
    context = flutterPluginBinding.applicationContext
    channel = MethodChannel(flutterPluginBinding.binaryMessenger, "flutter_bk_ua")
    channel.setMethodCallHandler(this)
  }

  override fun onMethodCall(@NonNull call: MethodCall, @NonNull result: Result) {
    if (call.method == "getUserAgent") {
      val pkgInfo = getPkgInfo()
      val uiManager = context.getSystemService(Context.UI_MODE_SERVICE) as UiModeManager
      return result.success(mapOf(
        "platform" to "Android",
        "platformVersion" to Build.VERSION.RELEASE,
        "model" to Build.MODEL,
        "mobile" to (uiManager.currentModeType == Configuration.UI_MODE_TYPE_NORMAL),
        "device" to Build.DEVICE,
        "architecture" to Build.SUPPORTED_ABIS[0],
        "brand" to pkgInfo?.applicationInfo?.loadLabel(context.packageManager)?.toString(),
        "appVersion" to pkgInfo?.versionName,
        "packageName" to pkgInfo?.applicationInfo?.packageName,
        "appName" to pkgInfo?.applicationInfo?.packageName
      ))
    }

    result.notImplemented()
  }

  fun getPkgInfo (): PackageInfo? {
    val info: PackageInfo? = try {
      context.packageManager.getPackageInfo(context.packageName, 0)
    } catch (e: PackageManager.NameNotFoundException) {
      null
    }
    return info
  }

  override fun onDetachedFromEngine(@NonNull binding: FlutterPlugin.FlutterPluginBinding) {
    channel.setMethodCallHandler(null)
  }
}
