import Flutter
import UIKit

public class SwiftFlutterBkUAPlugin: NSObject, FlutterPlugin {
  public static func register(with registrar: FlutterPluginRegistrar) {
    let channel = FlutterMethodChannel(name: "flutter_bk_ua", binaryMessenger: registrar.messenger())
    let instance = SwiftFlutterBkUAPlugin()
    registrar.addMethodCallDelegate(instance, channel: channel)
  }

  public func handle(_ call: FlutterMethodCall, result: @escaping FlutterResult) {
    if (call.method == "getUserAgent") {
      let currentDevice = UIDevice.current;
      
      return result([
        "platform": currentDevice.systemName,
        "platformVersion": currentDevice.systemVersion,
        "model": currentDevice.model,
        "mobile": isMobile(userInterfaceIdiom: currentDevice.userInterfaceIdiom),
        "device": getModelName(),
        "architecture": cpuType(),
        "packageName": Bundle.main.bundleIdentifier,
        "appName": getDictionaryVal(key: "CFBundleDisplayName"),
        "appVersion": getDictionaryVal(key: "CFBundleShortVersionString"),
        "buildNumber": getDictionaryVal(key: "CFBundleVersion"),
      ])
    }

    result(FlutterMethodNotImplemented);
  }

  func isMobile (userInterfaceIdiom: UIUserInterfaceIdiom) -> Bool {
    return userInterfaceIdiom == .phone || userInterfaceIdiom == .pad
  }

  func getDictionaryVal (key: String) -> String {
      return (Bundle.main.object(forInfoDictionaryKey: key) ?? "") as! String
  }

  func cpuType() -> String {
    var type = cpu_type_t()
    var cpuSize = MemoryLayout<cpu_type_t>.size
    sysctlbyname("hw.cputype", &type, &cpuSize, nil, 0)

    var subType = cpu_subtype_t()
    var subTypeSize = MemoryLayout<cpu_subtype_t>.size
    sysctlbyname("hw.cpusubtype", &subType, &subTypeSize, nil, 0)

    switch type {
      case CPU_TYPE_X86_64:
        switch subType {
        case CPU_SUBTYPE_X86_64_H: return "x86_64h"
        case CPU_SUBTYPE_X86_ARCH1: return "x86_arch1"
        case CPU_SUBTYPE_X86_64_ALL: return "x86_64"
        default: return "x86_64"
        }
      case CPU_TYPE_X86: return "x86"

      case CPU_TYPE_ARM:
        switch subType {
        case CPU_SUBTYPE_ARM_V8: return "armv8"
        case CPU_SUBTYPE_ARM_V7: return "armv7"
        case CPU_SUBTYPE_ARM_V7EM: return "armv7em"
        case CPU_SUBTYPE_ARM_V7F: return "armv7f"
        case CPU_SUBTYPE_ARM_V7K: return "armv7k"
        case CPU_SUBTYPE_ARM_V7M: return "armv7m"
        case CPU_SUBTYPE_ARM_V7S: return "armv7s"
        case CPU_SUBTYPE_ARM_V6: return "armv6"
        case CPU_SUBTYPE_ARM_V6M: return "armv6m"
        case CPU_SUBTYPE_ARM_V4T: return "armv4t"
        case CPU_SUBTYPE_ARM_V5TEJ: return "armv5"
        case CPU_SUBTYPE_ARM_XSCALE: return "xscale"
        case CPU_SUBTYPE_ARM_ALL: return "arm"
        default: return "arm"
        }

      case CPU_TYPE_ARM64:
        switch subType {
        case CPU_SUBTYPE_ARM64_V8: return "arm64v8"
        case CPU_SUBTYPE_ARM64E: return "arm64e"
        case CPU_SUBTYPE_ARM64_ALL: return "arm64"
        default: return "arm64"
        }

      case CPU_TYPE_ARM64_32: return "arm64_32"

      case CPU_TYPE_POWERPC: return "ppc"
      case CPU_TYPE_POWERPC64: return "ppc64"
      case CPU_TYPE_VAX: return "vax"
      case CPU_TYPE_I860: return "i860"
      case CPU_TYPE_I386: return "i386"
      case CPU_TYPE_HPPA: return "hppa"
      case CPU_TYPE_SPARC: return "sparc"
      case CPU_TYPE_MC88000: return "m88k"

      case CPU_TYPE_ANY: return "any"
      default: return "unknown"
    }
  }

  func getModelName () -> String {
      var sysInfo = utsname()
      uname(&sysInfo)
      let machine = String(bytes: Data(bytes: &sysInfo.machine, count: Int(_SYS_NAMELEN)), encoding: .ascii)!.trimmingCharacters(in: .controlCharacters)

      switch machine {
      case "iPhone3,1", "iPhone3,2", "iPhone3,3":  return "iPhone 4"
      case "iPhone4,1":  return "iPhone 4s"
      case "iPhone5,1":   return "iPhone 5"
      case  "iPhone5,2":  return "iPhone 5 (GSM+CDMA)"
      case "iPhone5,3":  return "iPhone 5c (GSM)"
      case "iPhone5,4":  return "iPhone 5c (GSM+CDMA)"
      case "iPhone6,1":  return "iPhone 5s (GSM)"
      case "iPhone6,2":  return "iPhone 5s (GSM+CDMA)"
      case "iPhone7,2":  return "iPhone 6"
      case "iPhone7,1":  return "iPhone 6 Plus"
      case "iPhone8,1":  return "iPhone 6s"
      case "iPhone8,2":  return "iPhone 6s Plus"
      case "iPhone8,4":  return "iPhone SE"
      case "iPhone9,1":   return "国行、日版、港行iPhone 7"
      case "iPhone9,2":  return "港行、国行iPhone 7 Plus"
      case "iPhone9,3":  return "美版、台版iPhone 7"
      case "iPhone9,4":  return "美版、台版iPhone 7 Plus"
      case "iPhone10,1","iPhone10,4":   return "iPhone 8"
      case "iPhone10,2","iPhone10,5":   return "iPhone 8 Plus"
      case "iPhone10,3","iPhone10,6":   return "iPhone X"
      case "iPhone11,2": return "iPhone XS"
      case "iPhone11,4", "iPhone11,6": return "iPhone XS Max"
      case "iPhone11,8": return "iPhone XR"
      case "iPhone12,1": return "iPhone 11"
      case "iPhone12,3": return "iPhone 11 Pro"
      case "iPhone12,5": return "iPhone 11 Pro Max"
      case "iPhone13,1": return "iPhone 12 mini"
      case "iPhone13,2": return "iPhone 12"
      case "iPhone13,3": return "iPhone 12  Pro"
      case "iPhone13,4": return "iPhone 12  Pro Max"
      case "iPhone14,2": return "iPhone 13 Pro"
      case "iPhone14,3": return "iPhone 13 Pro Max"
      case "iPhone14,4": return "iPhone 13 Mini"
      case "iPhone14,5": return "iPhone 13"
      case "i386", "x86_64":   return "Simulator"
      default:  return machine
      }
  }
}
