import 'package:flutter/services.dart';
import 'package:flutter_bk_ua/flutter_bk_ua.dart';
import 'package:flutter_test/flutter_test.dart';

final testUaModel = UAModel(
  platform: 'iOS',
  platformVersion: '13.2',
  architecture: 'arm64',
  model: 'iPhone',
  mobile: true,
  device: 'iphone 12 pro',
  appName: 'sampleApp',
  packageName: 'com.xxx',
  appVersion: '1.0.0',
);
void main() {
  const MethodChannel channel = MethodChannel('flutter_bk_ua');

  TestWidgetsFlutterBinding.ensureInitialized();

  setUp(() {
    channel.setMockMethodCallHandler((MethodCall methodCall) async {
      return testUaModel.toJSON();
    });
  });

  tearDown(() {
    channel.setMockMethodCallHandler(null);
  });

  test('getUserAgent', () async {
    final ua = await getUserAgent();
    print(ua.toJSON());
    expect(ua.toJSON(), testUaModel.toJSON());
  });

  test('getUAHeader', () async {
    final String delimiter = ';';
    final uaHeader = await getUAHeader(delimiter: delimiter);
    print(uaHeader);
    expect(
        uaHeader,
        [
          testUaModel.appName,
          testUaModel.packageName,
          testUaModel.appVersion,
          testUaModel.device,
          '${testUaModel.platform} ${testUaModel.platformVersion}',
        ].join(delimiter));
  });
}
